@@ -8,12 +8,10 @@
  *           it must have been created beforehand.  msql-import
  *           simply sends INSERTs to the database server.
  *
- *           When importing a flat fiel that was created in
+ *           When importing a flat file that was created in
  *           the DOS world, make sure you convert the file
  *           to unix by replacing CR/LF with LF only.
  *
- * Date:     March 1996
- *
  * Author:   Pascal Forget <pascal@wsc.com>
  *
  * Copyright (C) 1995-1996 Pascal Forget.  All Rights Reserved
@@ -31,7 +29,7 @@
 #include "msql-import.h"
 #include <string.h>
 
-#define MSQL_IMPORT_VERSION "0.0.9"
+#define MSQL_IMPORT_VERSION "0.1.2"
 #define IMPORT_MAX_FIELDS 255
 #define MAX(a,b) (a>b)a:b
 
@@ -41,158 +39,6 @@ int sock;
 
 SafeString *query;
 
-/*****************************************************************
- * _append_character() appends a character to str                *
- *****************************************************************/
-
-void
-append_character(SafeString *str, char c)
-{
-    int length = 0;
-    
-    if (str[0].buffer != NULL)
-    {
-	length = strlen(str[0].buffer);
-    }
-    
-    set_string_capacity(str, length + 1);
-
-    str[0].buffer[length] = c;
-    str[0].buffer[length+1] = '\0';
-}
-
-/*****************************************************************
- * _append_string_buffer() appends a string buffer to str        *
- *****************************************************************/
-
-void
-append_string_buffer(SafeString *str, const char *newBuffer)
-{
-    if (newBuffer != NULL)
-    {
-	if (str[0].buffer != NULL)
-	{
-	    set_string_capacity(str,strlen(str[0].buffer)+strlen(newBuffer));
-	}
-	else
-	{
-	    set_string_capacity(str, strlen(newBuffer));
-	}
-	    
-	if (str[0].buffer != NULL)
-	{
-	    strcat(str[0].buffer, newBuffer);
-	}
-	else
-	{
-	    strcpy(str[0].buffer, newBuffer);
-	}
-    }
-}
-
-/*****************************************************************
- * _copy_string_buffer() copies a string buffer in str           *
- *****************************************************************/
-
-void
-copy_string_buffer(SafeString *str, const char *newBuffer)
-{
-    if (newBuffer != NULL)
-    {
-	set_string_capacity(str, strlen(newBuffer));
-	strcpy(str[0].buffer, newBuffer);
-    }
-    else
-    {
-	str[0].buffer = NULL;
-    }
-}
-
-/*****************************************************************
- * _create_string() allocates and initializes a SafeString       *
- *****************************************************************/
-
-SafeString *
-create_string(void)
-{
-    SafeString *string = (SafeString *)malloc(sizeof(SafeString));
-    string[0].buffer = (char *)NULL;
-    string[0].capacity = 0;
-    return string;
-}
-
-/*****************************************************************
- * _set_string_buffer() sets the String's buffer                 *
- *****************************************************************/
-
-void
-set_string_buffer(SafeString *str, char *newBuffer)
-{
-    str[0].buffer = newBuffer;
-
-    if (str[0].buffer != NULL)
-    {
-	str[0].capacity = strlen(newBuffer);
-    }
-    else
-    {
-	str[0].capacity = 0;
-    }
-}
-
-/*****************************************************************
- * _set_string_capacity() allocates a bigger buffer if needed    *
- *****************************************************************/
-
-void
-set_string_capacity(SafeString *str, unsigned int cap)
-{
-    if (str[0].capacity < cap)
-    {
-	if (str[0].buffer != NULL)
-	{
-	    str[0].buffer = (char *)realloc(str[0].buffer, cap+1);
-	}
-	else
-	{
-	    str[0].buffer = (char *)malloc(cap+1);
-	    str[0].buffer[0] = '\0';
-	}
-	str[0].capacity = cap;
-    }
-}
-
-/*****************************************************************
- * _safe_c_string_copy() makes a copy of a string and returns    *
- * the new string.                                               *
- *                                                               *
- * This function replaces strdup() because:                      *
- *      Not all Unixes implement strdup                          *
- *      Some strdup implementations crash on a NULL argument     *
- *****************************************************************/
-
-char *
-safe_c_string_copy(const char *s)
-{
-    char *s1;
-    
-    if (s == NULL)
-    {
-	return(NULL);
-    }
-    
-    s1 = (char *)malloc(strlen(s) + 1);
-    
-    if (s1 == NULL)
-    {
-	return(NULL);
-    }
-    
-    (void)strcpy(s1,s);
-    
-    return(s1);
-}
-
 /**********************************************************************/
 /* File Id:                     strparse                              */
 /* Author:                      Stan Milam.                           */
@@ -277,27 +123,6 @@ alarm_msg(const char *message)
     abort_import(-1);
 }
 
-/*****************************************************************
- * _append_string() returns the string s, concatenated with the  *
- * append string.  The returned string is not garanteed to be    *
- * at the same memory location as s was.                         *
- *****************************************************************/
-
-char *
-append_string(char *s, const char *append)
-{
-    int newLength;
-    
-    if ((append!= NULL) && ((newLength = strlen(append)) > 0))
-    {
-	s = realloc(s, strlen(s) + newLength + 1);
-
-	strcat(s, append);
-    }
-
-    return s;
-}
-
 /*****************************************************************
  * Put a backslash in front of single quote, parentheses,        *
  * backslashes, and other special characters in the string. This *
@@ -368,6 +193,29 @@ backslashify_special_chars(const char *s)
     
     /* Do the other special characters */
     
+#if 1
+    while(special_chars[current])
+    {
+	index = -2;
+
+	while ((ptr = strchr(str+index+2, special_chars[current])) != NULL)
+	{
+	    len = strlen(str);
+	    index = ptr-str;
+	    str = (char *)realloc(str, len+2);
+	    
+	    for (i=len; i>=index; i--)
+	    {
+		str[i+1] = str[i];
+	    }
+	    
+	    str[index] = BACKSLASH_CHAR;
+	}
+	current++;
+    }
+
+#else /* the following chunk of code is buggy */
+
     while(special_chars[current])
     {
 	index = -2;
@@ -387,7 +235,7 @@ backslashify_special_chars(const char *s)
 	}
 	current++;
     }
-
+#endif
     return str;
 }
 
@@ -499,8 +347,8 @@ get_record(FILE *fp, const char record_delimiter, SafeString *record)
     char c;
     int i = 0;
 
-    set_string_capacity(record, GET_REC_BLOCKSIZE);
-    record[0].buffer[0] = '\0';
+    ss_set_string_capacity(record, GET_REC_BLOCKSIZE);
+    ss_copy_c_string(record, "");
     
     for ( ; ; )
     {
@@ -511,7 +359,7 @@ get_record(FILE *fp, const char record_delimiter, SafeString *record)
 	else
 	{
 	    /* add the block to the record string */
-	    append_character(record, c);
+	    ss_append_character(record, c);
 	    i++;
 	}
     }
@@ -549,9 +397,9 @@ import_file(char *table,
 	abort_import(-1);
     }
 
-    record = create_string();
-    query = create_string();
-    set_string_capacity(query, GET_REC_BLOCKSIZE);
+    record = ss_create_string();
+    query = ss_create_string();
+    ss_set_string_capacity(query, GET_REC_BLOCKSIZE);
     
     while (get_record(fp, rowDel[0], record) != NULL)
     {
@@ -563,12 +411,12 @@ import_file(char *table,
 			      fieldDel);
     }
 
-    free(fieldDel);
-    free(rowDel);
-    free(table);
-    free(record);
-    free(query);
-    free(path);
+    safe_c_string_free(fieldDel);
+    safe_c_string_free(rowDel);
+    safe_c_string_free(table);
+    ss_destroy_string(record);
+    ss_destroy_string(query);
+    safe_c_string_free(path);
     
     if (recs > 1)
     {
@@ -625,20 +473,19 @@ insert_record(const char *table,
     int fieldsCount = types[0]; /* The number of values to be inserted */
     
     sprintf(delimiter, "%s", fieldDel);
-    query[0].buffer[0] = '\0';
 
-    copy_string_buffer(query, "INSERT INTO ");
-    append_string_buffer(query, table);
+    ss_copy_c_string(query, "INSERT INTO ");
+    ss_append_c_string(query, table);
     
     if ((fields == NULL) || (strlen(fields) < 2))
     {
-	append_string_buffer(query, " VALUES(");
+	ss_append_c_string(query, " VALUES(");
     }
     else
     {
-	append_string_buffer(query, " (");
-	append_string_buffer(query, fields);
- 	append_string_buffer(query, ") VALUES(");
+	ss_append_c_string(query, " (");
+	ss_append_c_string(query, fields);
+ 	ss_append_c_string(query, ") VALUES(");
     }
     
     if((record != NULL) && (token = str_parse(ptr,delimiter)))
@@ -647,9 +494,9 @@ insert_record(const char *table,
 	{
 	  case CHAR_TYPE:
 	    value = backslashify_special_chars(token);
-	    append_character(query, '\'');
-	    append_string_buffer(query, value);
-	    append_string_buffer(query,"',");
+	    ss_append_character(query, '\'');
+	    ss_append_c_string(query, value);
+	    ss_append_c_string(query,"',");
 	    
 	    if (value != NULL)
 	    {
@@ -659,12 +506,12 @@ insert_record(const char *table,
 	    break;
 
 	  case NULL_TYPE:
-	    append_string_buffer(query, "NULL,");
+	    ss_append_c_string(query, "NULL,");
 	    break;
 
 	  default:
-	    append_string_buffer(query, token);
-	    append_character(query,',');
+	    ss_append_c_string(query, token);
+	    ss_append_character(query,',');
 	}
     }
     else
@@ -680,23 +527,21 @@ insert_record(const char *table,
 	    {
 	      case CHAR_TYPE:        
 		value = backslashify_special_chars(token);
-		append_character(query, '\'');
-		append_string_buffer(query, value);
-		append_string_buffer(query,"',");
-		
-		if (value!=NULL)
-		{
-		    free(value);
-		}
+		ss_append_character(query, '\'');
+		ss_append_c_string(query, value);
+		ss_append_c_string(query,"',");
+
+		safe_c_string_free(value);
+
 		break;
 		
 	      case NULL_TYPE:
-		append_string_buffer(query, "NULL,");
+		ss_append_c_string(query, "NULL,");
 		break;
 		
 	      default:
-		append_string_buffer(query, token);
-		append_character(query, ',');
+		ss_append_c_string(query, token);
+		ss_append_character(query, ',');
 	    }
 	}
 	else
@@ -715,15 +560,16 @@ insert_record(const char *table,
     
     while (fieldNumber < fieldsCount)
     {
-	append_string_buffer(query, "NULL,");
+	ss_append_c_string(query, "NULL,");
 	fieldNumber++;
     }
 
-    
     /* Remove the last comma and the record delimiter ('\n') */
     query[0].buffer[strlen(query[0].buffer)-1] = '\0';
+    query[0].length--;
     
-    append_character(query, ')');
+    ss_append_character(query, ')');
+    fprintf(stdout, "Query : %s\n\n", query[0].buffer);
 
     if (msqlQuery(sock, query[0].buffer) == -1)
     {
@@ -1034,4 +880,6 @@ main(int argc, char **argv)
 		format_delimiter(col_delimiter),
 		format_delimiter(rec_delimiter),
 		fields);
+
+    abort_import(0); /* close socket and quit, we're done. */
 }
