@@ -8,7 +8,7 @@
  *           it must have been created beforehand.  msql-import
  *           simply sends INSERTs to the database server.
  *
- *           When importing a flat fiel that was created in
+ *           When importing a flat file that was created in
  *           the DOS world, make sure you convert the file
  *           to unix by replacing CR/LF with LF only.
  *
@@ -32,24 +32,11 @@
 #include "msql.h"
 #include <stdio.h>
 #include <stdlib.h>
-
-typedef struct {
-    char *buffer;
-    unsigned int capacity;
-} SafeString;
-
-/* SafeString functions */
-void append_character(SafeString *str, char c);
-void append_string_buffer(SafeString *str, const char *newBuffer);
-void copy_string_buffer(SafeString *str, const char *newBuffer);
-SafeString *create_string(void);
-void set_string_buffer(SafeString *str, char *newBuffer);
-void set_string_capacity(SafeString *str, unsigned int cap);
+#include "safestring.h"
 
 void abort_import(int exitCode);
 void alarm_msql(void);
 void alarm_msg(const char *message);
-char *append_string(char *s, const char *append);
 char *backslashify_special_chars(const char *s);
 int *datatypes(const char *table, const char *fields);
 char *format_delimiter(const char *str);
